@@ -30,19 +30,10 @@ from pdm.pep517.utils import (
     safe_name,
 )
 from pdm.pep517.validator import validate_pep621
+from pdm.pep517.versions import AVAILABLE_PYTHON_VERSIONS
 
 T = TypeVar("T")
 
-AVAILABLE_PYTHON_VERSIONS = (
-    "2.7",
-    "3.4",
-    "3.5",
-    "3.6",
-    "3.7",
-    "3.8",
-    "3.9",
-)
-
 
 class ProjectError(ValueError):
     pass
@@ -67,6 +58,20 @@ class MetaField(Generic[T]):
             return None
 
 
+def _make_version_collections(python_versions: List[str]) -> Dict[str, List[Version]]:
+    rv: Dict[str, List[Version]] = {}
+    for raw in python_versions:
+        version = Version(raw)
+        if version.minor == 0:
+            key = str(version.major)
+        else:
+            key = "{0.major}.{0.minor}".format(version)
+
+        rv.setdefault(key, []).append(version)
+
+    return rv
+
+
 class Metadata:
     """A class that holds all metadata that Python packaging requries."""
 
@@ -227,8 +232,12 @@ class Metadata:
                 else SpecifierSet()
             )
 
-            for version in AVAILABLE_PYTHON_VERSIONS:
-                if python_constraint.contains(Version(version)):
+            version_collections = _make_version_collections(AVAILABLE_PYTHON_VERSIONS)
+            for version, micro_versions in version_collections.items():
+                if any(
+                    python_constraint.contains(micro_version)
+                    for micro_version in micro_versions
+                ):
                     classifers.add(f"Programming Language :: Python :: {version[0]}")
                     classifers.add(f"Programming Language :: Python :: {version}")
 
@@ -280,6 +289,14 @@ class Metadata:
             return "src"
         return ""
 
+    @property
+    def editable_backend(self) -> str:
+        """Currently only two backends are supported:
+        - editables: Proxy modules via editables(default)
+        - path: the legacy .pth file method
+        """
+        return self._tool_settings.get("editable-backend", "editables")
+
     def _convert_dependencies(self, deps: List[str]) -> List[str]:
         return list(filter(None, map(ensure_pep440_req, deps)))
 
@@ -316,7 +333,8 @@ class Metadata:
 
     @property
     def requires_python(self) -> str:
-        return self._metadata.get("requires-python", "")
+        result = self._metadata.get("requires-python", "")
+        return "" if result == "*" else result
 
     @property
     def entry_points(self) -> Dict[str, List[str]]:
