@@ -1,34 +1,18 @@
-import contextlib
-import tarfile
+import email
 import zipfile
 
 import pytest
 
-from pdm.pep517 import api, utils
-from tests import FIXTURES
-
-
-def get_tarball_names(path):
-    with tarfile.open(path, "r:gz") as tar:
-        return tar.getnames()
-
-
-def get_wheel_names(path):
-    with zipfile.ZipFile(path) as zf:
-        return zf.namelist()
-
-
-@contextlib.contextmanager
-def build_fixture_project(project_name):
-    project = FIXTURES / "projects" / project_name
-    with utils.cd(project):
-        yield
+from pdm.pep517 import api
+from tests.testutils import build_fixture_project, get_tarball_names, get_wheel_names
 
 
 def test_build_single_module(tmp_path):
     with build_fixture_project("demo-module"):
         wheel_name = api.build_wheel(tmp_path.as_posix())
         sdist_name = api.build_sdist(tmp_path.as_posix())
+        assert api.get_requires_for_build_sdist() == []
+        assert api.get_requires_for_build_wheel() == []
         assert sdist_name == "demo-module-0.1.0.tar.gz"
         assert wheel_name == "demo_module-0.1.0-py3-none-any.whl"
         tar_names = get_tarball_names(tmp_path / sdist_name)
@@ -55,7 +39,7 @@ def test_build_package(tmp_path):
         wheel_name = api.build_wheel(tmp_path.as_posix())
         sdist_name = api.build_sdist(tmp_path.as_posix())
         assert sdist_name == "demo-package-0.1.0.tar.gz"
-        assert wheel_name == "demo_package-0.1.0-py3-none-any.whl"
+        assert wheel_name == "demo_package-0.1.0-py2.py3-none-any.whl"
 
         tar_names = get_tarball_names(tmp_path / sdist_name)
         assert "demo-package-0.1.0/my_package/__init__.py" in tar_names
@@ -184,6 +168,8 @@ def test_build_with_cextension(tmp_path):
     with build_fixture_project("demo-cextension"):
         wheel_name = api.build_wheel(tmp_path.as_posix())
         sdist_name = api.build_sdist(tmp_path.as_posix())
+        assert api.get_requires_for_build_sdist() == []
+        assert api.get_requires_for_build_wheel() == ["setuptools>=40.8.0"]
 
         zip_names = get_wheel_names(tmp_path / wheel_name)
         assert "my_package/__init__.py" in zip_names
@@ -220,3 +206,70 @@ def test_build_with_cextension_in_src(tmp_path):
         assert not any(
             path.startswith("build") for path in tar_names
         ), 'Not collect c files in temporary directory "./build"'
+
+
+def test_build_editable(tmp_path):
+    with build_fixture_project("demo-package") as project:
+        wheel_name = api.build_editable(tmp_path.as_posix())
+        assert api.get_requires_for_build_editable() == []
+        with zipfile.ZipFile(tmp_path / wheel_name) as zf:
+            namelist = zf.namelist()
+            assert "demo_package.pth" in namelist
+            assert "_demo_package.py" in namelist
+
+            metadata = email.message_from_bytes(
+                zf.read("demo_package-0.1.0.dist-info/METADATA")
+            )
+            assert "editables" in metadata.get_all("Requires-Dist", [])
+
+            pth_content = zf.read("demo_package.pth").decode("utf-8").strip()
+            assert pth_content == "import _demo_package"
+
+            proxy_module = zf.read("_demo_package.py").decode("utf-8").strip()
+            assert proxy_module == (
+                "from editables.redirector import RedirectingFinder as F\n"
+                "F.install()\n"
+                "F.map_module('my_package', {0!r})".format(
+                    str((project / "my_package" / "__init__.py").resolve())
+                )
+            )
+
+
+def test_build_editable_src(tmp_path):
+    with build_fixture_project("demo-src-package") as project:
+        wheel_name = api.build_editable(tmp_path.as_posix())
+
+        with zipfile.ZipFile(tmp_path / wheel_name) as zf:
+            namelist = zf.namelist()
+            assert "demo_package.pth" in namelist
+            assert "_demo_package.py" in namelist
+
+            pth_content = zf.read("demo_package.pth").decode("utf-8").strip()
+            assert pth_content == "import _demo_package"
+
+            proxy_module = zf.read("_demo_package.py").decode("utf-8").strip()
+            assert proxy_module == (
+                "from editables.redirector import RedirectingFinder as F\n"
+                "F.install()\n"
+                "F.map_module('my_package', {0!r})".format(
+                    str((project / "src" / "my_package" / "__init__.py").resolve())
+                )
+            )
+
+
+def test_build_editable_pep420(tmp_path):
+    with build_fixture_project("demo-pep420-package") as project:
+        wheel_name = api.build_editable(tmp_path.as_posix())
+
+        with zipfile.ZipFile(tmp_path / wheel_name) as zf:
+            namelist = zf.namelist()
+            assert "demo_package.pth" in namelist
+            assert "_demo_package.py" not in namelist
+
+            metadata = email.message_from_bytes(
+                zf.read("demo_package-0.1.0.dist-info/METADATA")
+            )
+            assert "editables" not in metadata.get_all("Requires-Dist", [])
+
+            pth_content = zf.read("demo_package.pth").decode("utf-8").strip()
+            assert pth_content == str(project.resolve())
